import os
import datetime as dt
import re
import pickle
import csv
from os import listdir
from os.path import isfile, join
import datetime as dt

class FileMatchException(Exception):
    def __init__(self, message):
        super().__init__(message)
        
'''
store data collected in files for potential later use
INPUT: start_t- string that represents the start date of data collection. end_t- string that represents the end date of data collection
        json_max- int number of tweets collected per interval. interval_len- int that represents number of minutes in interval
'''
def save_lists(json_response_list, json_sample_list, sentiment_list, sentiment_sample, start_t, end_t, new_end_t, json_max, interval_len):
    #string format of the time at the current moment
    time_now = dt.datetime.now().isoformat()
    
    #get path to parent directory of this file
    rel_path = os.path.dirname(os.path.realpath(__file__))

    #construct the full path to store collected tweet data
    datadir = "storedqueries"
    fullpath = os.path.join(rel_path, datadir)
    
    #remove illegal characters from filename
    tweetfile = "tweet_" + start_t + "_" + end_t + "_" + str(json_max) + "_" + str(interval_len) + "_"
    tweetfile = re.sub(r":",".",tweetfile)
    tweetfile = re.sub(r"/",".",tweetfile)
    tweetfile = re.sub(r"\.",".",tweetfile)
    sample_tweetfile = tweetfile
    tweetfile += ".csv"
    sample_tweetfile += "sample.csv"
    os.makedirs(os.path.abspath(fullpath), mode=0o666, exist_ok=True)
    
    #make new filename for updated enddate
    newtweetfile = "tweet_" + start_t + "_" + new_end_t + "_" + str(json_max) + "_" + str(interval_len) + "_"
    newtweetfile = re.sub(r":",".",newtweetfile)
    newtweetfile = re.sub(r"/",".",newtweetfile)
    newtweetfile = re.sub(r"\.",".",newtweetfile)
    new_sample_tweetfile = newtweetfile
    newtweetfile += ".csv"
    new_sample_tweetfile += "sample.csv"
    
    #remove illegal characters from filename
    sentifile = "senti_" + start_t + "_" + end_t + "_" + str(json_max) + "_" + str(interval_len) + "_"
    sentifile = re.sub(r":",".",sentifile)
    sentifile = re.sub(r"/",".",sentifile)
    sentifile = re.sub(r"\.",".",sentifile)
    sample_sentifile = sentifile
    sentifile += ".csv"
    sample_sentifile += "sample.csv"
    
    #make new filename for updated enddate
    newsentifile = "senti_" + start_t + "_" + new_end_t + "_" + str(json_max) + "_" + str(interval_len) + "_"
    newsentifile = re.sub(r":",".",newsentifile)
    newsentifile = re.sub(r"/",".",newsentifile)
    newsentifile = re.sub(r"\.",".",newsentifile)
    new_sample_sentifile = newsentifile
    newsentifile += ".csv"
    new_sample_sentifile += "sample.csv"
    
    #store tweet text array if needed
    with open(os.path.join(fullpath,tweetfile), "a", newline='') as csvfile:
        tweetwriter = csv.writer(csvfile, delimiter=',',quotechar='|', quoting=csv.QUOTE_MINIMAL)
        for row in json_response_list:
            tweetwriter.writerow(row)
    with open(os.path.join(fullpath,sample_tweetfile), "a", newline='') as csvfile:
        tweetwriter = csv.writer(csvfile, delimiter=',',quotechar='|', quoting=csv.QUOTE_MINIMAL)
        for row in json_sample_list:
            tweetwriter.writerow(row)
            
    #store tweet sentiment array if needed
    with open(os.path.join(fullpath,sentifile), "a", newline='') as csvfile:
        sentiwriter = csv.writer(csvfile, delimiter=',',quotechar='|', quoting=csv.QUOTE_MINIMAL)
        for row in sentiment_list:
            sentiwriter.writerow(row)
    with open(os.path.join(fullpath,sample_sentifile), "a", newline='') as csvfile:
        sentiwriter = csv.writer(csvfile, delimiter=',',quotechar='|', quoting=csv.QUOTE_MINIMAL)
        for row in sentiment_sample:
            sentiwriter.writerow(row)
    
    #check if tweet and sentiment files with matching parameters exists
    if not (os.path.exists(os.path.join(fullpath,tweetfile)) and os.path.exists(os.path.join(fullpath,sentifile))):
        raise FileMatchException("No matching tweet and sentiment files found")
        
    #rename file to reflect updated enddate
    if os.path.exists(os.path.join(fullpath,tweetfile)) and tweetfile != newtweetfile:
        os.rename(os.path.join(fullpath,tweetfile), os.path.join(fullpath,newtweetfile))
        
    if os.path.exists(os.path.join(fullpath,sample_tweetfile)) and tweetfile != new_sample_tweetfile:
        os.rename(os.path.join(fullpath,sample_tweetfile), os.path.join(fullpath,new_sample_tweetfile))
        
    #rename file to reflect updated enddate
    if os.path.exists(os.path.join(fullpath,sentifile)) and sentifile != newsentifile:
        os.rename(os.path.join(fullpath,sentifile), os.path.join(fullpath,newsentifile))
        
    if os.path.exists(os.path.join(fullpath,sample_sentifile)) and sentifile != new_sample_sentifile:
        os.rename(os.path.join(fullpath,sample_sentifile), os.path.join(fullpath,new_sample_sentifile))
        
'''
INPUT: prefix- string representing the file prefix to search for
        json_max- max number of results stored in file for each time interval
        interval_len- int that represents length of collection intervals in minutes
        has_sample- boolean that represents whether the file is a comparison sample or the data specified in the query parameters
OUTPUT: tuple that contains the string file name that was searched for, the string start and end date of data storage and the int length of data collection in minutes
'''
def find_file(onlyfiles, prefix, json_max, interval_len, has_sample):
    past_file = ""

    #search in specified directory for sentiment file that matches specified data storage file values (which are contained within the file name)
    for file in onlyfiles:
        file_data = file.split("_")
        file_prefix = file_data[0]
        if file_prefix == "":
            continue
        file_startdate = file_data[1]
        datestr = file_startdate
        date_s = file_startdate.split(".")
        file_enddate = file_data[2]
        datestr2 = file_enddate
        date_c = file_enddate.split(".")
        file_json_max = int(file_data[3])
        file_interval_len = int(file_data[4])
        if ((file_prefix == prefix) and file_json_max == json_max and file_interval_len == interval_len):
            if has_sample:
                if "sample" in file:
                    past_file = file
                    break
            else:
                if "sample" not in file:
                    past_file = file
                    break
                    
    if past_file == "":
        raise FileMatchException("No sentiment storage files found")
        
    #create dates that represent the start and stop dates of file data storage
    start_date = dt.date(int(date_s[2])+2000, int(date_s[0]), int(date_s[1])) #using the date values from tweet_date_s and tweet_date_c is arbitrary; you could also use the equivalent sentiment values
    end_date = dt.date(int(date_c[2])+2000, int(date_c[0]), int(date_c[1]))
            
    #retrieve timezone-aware time at midnight
    ti = dt.time(tzinfo = dt.timezone.utc) 
        
    #make datetime from date and utc time 
    start_dt = dt.datetime.combine(start_date,ti) 
    end_dt = dt.datetime.combine(end_date,ti)
    
    #calculate totaltime from beginning of data collection to end
    totaltime = int((end_dt - start_dt) / dt.timedelta(minutes = 1))
        
    return (past_file, datestr, datestr2, totaltime)
        
def load_lists(json_max, interval_len):
    #get path to parent directory of this file
    rel_path = os.path.dirname(os.path.realpath(__file__))

    #construct the full path to store collected tweet data
    datadir = "storedqueries"
    mypath = os.path.join(rel_path, datadir)
            
    onlyfiles = [f for f in listdir(mypath) if isfile(join(mypath, f))]
    
    past_sentifile = ""
    past_tweetfile = ""
    senti_datestr = "a"
    tweet_datestr = "b"
    tweet_datestr2 = "c"
    senti_datestr2 = "d"
    
    if not onlyfiles:
        raise FileMatchException("No data storage files found")
        
    #retrieve file name with specified parameters
    past_sentifile, senti_datestr, senti_datestr2, _ = find_file(onlyfiles, 'senti', json_max, interval_len, False)
    past_tweetfile, tweet_datestr, tweet_datestr2, _ = find_file(onlyfiles, 'tweet', json_max, interval_len, False)
    
    #retrieve file name with specified parameters that contains sample comparison data
    past_sample_sentifile, sample_senti_datestr, sample_senti_datestr2, _ = find_file(onlyfiles, 'senti', json_max, interval_len, True)
    past_sample_tweetfile, sample_tweet_datestr, sample_tweet_datestr2, totaltime = find_file(onlyfiles, 'tweet', json_max, interval_len, True)
    
    if tweet_datestr != tweet_datestr or tweet_datestr2 != senti_datestr2:
        raise FileMatchException("No matching tweet and sentiment files found")
    else:
        sentiment_list = []
        if os.path.exists(os.path.join(mypath,past_sentifile)):
            #retrieve tweet sentiment array 
            with open(os.path.join(mypath,past_sentifile), "r", newline='') as csvfile:
                sentireader = csv.reader(csvfile, delimiter=',',quotechar='|', quoting=csv.QUOTE_MINIMAL)
                for row in sentireader:
                    sentiment_list.append([float(row[0]), float(row[1])])
        sentiment_sample = []
        if os.path.exists(os.path.join(mypath,past_sample_sentifile)):
            #retrieve tweet sentiment array 
            with open(os.path.join(mypath,past_sample_sentifile), "r", newline='') as csvfile:
                sentireader = csv.reader(csvfile, delimiter=',',quotechar='|', quoting=csv.QUOTE_MINIMAL)
                for row in sentireader:
                    sentiment_sample.append([float(row[0]), float(row[1])])

        tweet_list = []
        if os.path.exists(os.path.join(mypath,past_tweetfile)):
            #retrieve tweet sentiment array 
            with open(os.path.join(mypath,past_tweetfile), "r", newline='') as csvfile:
                tweetreader = csv.reader(csvfile, delimiter=',',quotechar='|', quoting=csv.QUOTE_MINIMAL)
                for row in tweetreader:
                    tweet_list.append(row)
        tweet_sample = []
        if os.path.exists(os.path.join(mypath,past_sample_tweetfile)):
            #retrieve tweet sentiment array 
            with open(os.path.join(mypath,past_sample_tweetfile), "r", newline='') as csvfile:
                tweetreader = csv.reader(csvfile, delimiter=',',quotechar='|', quoting=csv.QUOTE_MINIMAL)
                for row in tweetreader:
                    tweet_sample.append(row)

        return (sentiment_list,tweet_list,sentiment_sample, tweet_sample, totaltime)
    



import nltk
import re
from nltk.sentiment import SentimentIntensityAnalyzer
from nltk.tokenize import word_tokenize

nltk.download('stopwords')
nltk.download('vader_lexicon')
nltk.download('punkt')

'''
Sentiment is parsed from tweets
INPUT: interval_lists- list of lists that store cleaned tweet text for each interval
OUTPUT: all_sentiment- list of lists of identical dimensions to the input list that stores sentiment scores for each tweet
RESULTS: The sentiment parsed from the tweets can be used to gauge change in sentiment over time
'''
def parse(interval_lists):
    
    #initialize sentiment analysis tool
    sia = SentimentIntensityAnalyzer()
    #create lists to store sentiment analysis scores in a format that mimics the interval_lists fed as input to this method
    sent_list = [] #temp sublist of all_sentiment
    all_sentiment = [] #this list is returned
    
    #list of words that are meaningless to sentiment analysis
    stopwords = nltk.corpus.stopwords.words("english")
    
    #use all meaningful words in the strings within interval_lists to parse sentiment and store the sentiment within all_sentiment
    for interval in interval_lists:
        for tweet in interval:
            word_tokens = word_tokenize(tweet)
            words = [w + " " for w in word_tokens if w not in stopwords]
            words = "".join(words)
            #use the compound score to represent sentiment score
            sent_list.append(sia.polarity_scores(words)['compound'])
        all_sentiment.append(sent_list.copy())
        sent_list.clear()
        
    return all_sentiment
'''
sia vader 
Hutto, C.J. & Gilbert, E.E. (2014). VADER: A Parsimonious Rule-based Model for Sentiment Analysis of Social Media Text. Eighth International Conference on Weblogs and Social Media (ICWSM-14). Ann Arbor, MI, June 2014.
'''


import tkinter as tk
from tkcalendar import *
from tkinter import ttk
import datetime as dt
import math

class TwitterAPIArgumentError(Exception):
    def __init__(self, message):
        super().__init__(message)
        
class Cal_Impl:

    def __init__(self, enabled):
        #set up tkinter objects for initial display
        self.ws = tk.Tk()
        
        cal_label_frame = tk.Frame(self.ws)
        fzero = tk.Frame(self.ws)
        fone = tk.Frame(fzero)
        ftwo = tk.Frame(fzero)
        
        self.start_cal_label = tk.Label(
            cal_label_frame, 
            text="Select Start Date For Tweet Collection",
            font=("Times", 12),
            bg="#cae7e8"
            )
        self.end_cal_label = tk.Label(
            cal_label_frame, 
            text="Select End Date For Tweet Collection",
            font=("Times", 12),
            bg="#cae7e8"
            )
            
        fone.pack(side=tk.LEFT, padx=10, pady=10)
        ftwo.pack(side=tk.RIGHT, padx=10, pady=10)
        
        self.start_cal_label.pack(side=tk.LEFT, padx=10, pady=(2,5))
        self.end_cal_label.pack(side=tk.RIGHT, padx=10, pady=(2,5))
        
        cal_label_frame.pack(side=tk.TOP)
        cal_label_frame.config(bg="#cae7e8")
        
        fzero.pack(side=tk.TOP)
        fzero.config(bg="#cae7e8")
        
        #calculate maximum and minimum days that can be queried on Twitter Search API
        delta_day = dt.timedelta(days = 1)
        tw_start_date = dt.date(2006,3,21)
        
        self.start_cal = Calendar(
            fone, 
            selectmode="day", 
            mindate = tw_start_date,
            maxdate = dt.datetime.now() - delta_day,
            side=tk.LEFT
            )
        
        self.end_cal = Calendar(
            ftwo, 
            selectmode="day", 
            mindate = tw_start_date,
            maxdate = dt.datetime.now(),
            side=tk.RIGHT
            )
            
        self.msg = tk.Label(
            self.ws, 
            text="How long should data collection intervals be?",
            font=("Times", 12),
            bg="#cae7e8"
            )
        self.msg2 = tk.Label(
            self.ws, 
            text="How many tweets should be collected per interval?",
            font=("Times", 12),
            bg="#cae7e8"
            )
        self.msg_display = tk.Label(
            self.ws,
            text="",
            bg="#cae7e8"
        )
        
        #initialize combobox for interval length variable selection
        self.interval_bxstr = tk.StringVar(value = 240)
        self.interval_bx = ttk.Combobox(self.ws, textvariable=self.interval_bxstr)
        #self.interval_bx.bind('<<ComboboxSelected>>', interval_chosen)
        self.interval_bx['values'] = (60,240,1440)
        self.interval_bx.config(width = 15)
        
        #if default data collection is selected
        if not enabled:
            self.interval_bx.config(state=tk.DISABLED)
            
        #initialize combobox for interval length variable selection
        self.max_bxstr = tk.StringVar(value = 10)
        self.max_bx = ttk.Combobox(self.ws, textvariable=self.max_bxstr)
        #self.interval_bx.bind('<<ComboboxSelected>>', interval_chosen)
        self.max_bx['values'] = (10,50,100,1000)
        self.max_bx.config(width = 15)
        
        #if default data collection is selected
        if not enabled:
            self.max_bx.config(state=tk.DISABLED)
            
        #declare and initialize variables used to send data outside of the class
        self.end_dt = dt.datetime.now(dt.timezone.utc)
        self._totaltime = 0
        self._datestr2 = ""
        self._interval_len = 0
        self._datestr = ""
        self._json_max = 0
        self._has_values = False
        
    def on_quit(self):
        print("Calendar was terminated by exiting the window")
        self.ws.destroy()
        
    def display_msg(self):
        #retrieve selected start and end date from calendar  
        self._datestr = self.start_cal.get_date()
        
        self._datestr2 = self.end_cal.get_date()

        date_r = self._datestr.split("/")
        date2_r = self._datestr2.split("/")
        
        #adding the 2000 is bad practice, but this code will not exist in the year 3000 and twitter did not exist in 1999
        date = dt.date(int(date_r[2])+2000, int(date_r[0]), int(date_r[1]))
        date2 = dt.date(int(date2_r[2])+2000, int(date2_r[0]), int(date2_r[1]))
        
        #retrieve timezone-aware time at exactly midnight UTC
        ti = dt.time(tzinfo = dt.timezone.utc)
        
        self.start_dt = dt.datetime.combine(date,ti)
        self.end_dt = dt.datetime.combine(date2,ti)
        
        #retrieve use selections from comboboxes
        self._interval_len = self.interval_bx.get()
        self._json_max = self.max_bx.get()
        
        self._totaltime = (self.end_dt-self.start_dt)/dt.timedelta(minutes = 1)
        
        t = f"Your query is for tweets created between {self._datestr} and {self._datestr2} with data collection intervals of length {self._interval_len} minutes."
        print(t)
        
        #check for illegal arguments 
        if float(self._interval_len) < 1:
            raise TwitterAPIArgumentError(f"Invalid time interval ({self._interval_len}) received")
        pluralizer = "s" if float(self._interval_len) > 1 else ""
        if float(self._json_max) < 1:
            raise TwitterAPIArgumentError(f"Invalid rate of tweets ({self._json_max}) per ({self._interval_len}) minute{pluralizer} requested. At least one tweet must be requested per time interval.") 
        
        self._has_values = True
        #once user selects search parameters, UI is no longer necesssary
        self.ws.quit()
    
    #getter methods for user input
    @property   
    def interval_len(self):
        return self._interval_len
    @property
    def totaltime(self):
        return self._totaltime
    @property
    def datestr(self):
        return self._datestr
    @property
    def datestr2(self):
        return self._datestr2
    @property
    def json_max(self):
        return self._json_max
    @property
    def has_values(self):
        return self._has_values
        
    def run_cal(self):
        self.ws.title("Set Date Range for Data Collection")
        self.ws.geometry("600x430")
        self.ws.config(bg="#cae7e8")#819394 #cae7e8 borderwidth
        self.ws.config(borderwidth=15)
        
        self.start_cal.pack()
        self.end_cal.pack()

        actionBtn = tk.Button(
            self.ws,
            text="Run Query",
            command=self.display_msg
        )
        
        self.interval_bx.pack(side=tk.BOTTOM, expand=False, pady=2)
        self.msg.pack(side=tk.BOTTOM, pady=2)
        
        self.max_bx.pack(side=tk.BOTTOM, expand=False, pady=2)
        self.msg2.pack(side=tk.BOTTOM, pady=2)
        actionBtn.pack(side=tk.BOTTOM, pady=2)
        
        self.msg_display.pack(side=tk.BOTTOM, pady=3)
        
        self.ws.protocol('WM_DELETE_WINDOW', self.on_quit)
        self.ws.mainloop()
        
class Cal_End:

    def __init__(self, enabled):
        #set up tkinter objects for initial display
        self.ws = tk.Tk()
        
        #calculate maximum and minimum days that can be queried on Twitter Search API
        delta_day = dt.timedelta(days = 1)
        tw_start_date = dt.date(2006,3,21)

        self.end_cal = Calendar(
            self.ws, 
            selectmode="day", 
            mindate = tw_start_date,
            maxdate = dt.datetime.now()
            )
        self.msg = tk.Label(
            self.ws, 
            text="How long should data collection intervals be?",
            font=("Times", 12),
            bg="#cae7e8"
            )
        self.msg2 = tk.Label(
            self.ws, 
            text="How many tweets should be collected per interval?",
            font=("Times", 12),
            bg="#cae7e8"
            )
        self.msg_display = tk.Label(
            self.ws,
            text="",
            bg="#cae7e8"
        )
        
        #initialize combobox for interval length variable selection
        self.interval_bxstr = tk.StringVar(value = 240)
        self.interval_bx = ttk.Combobox(self.ws, textvariable=self.interval_bxstr)
        #self.interval_bx.bind('<<ComboboxSelected>>', interval_chosen)
        self.interval_bx['values'] = (60,240,1440)
        self.interval_bx.config(width = 15)
        
        #if default data collection is selected, parameter selection is unnecessary
        if not enabled:
            self.interval_bx.config(state=tk.DISABLED)
            
        #initialize combobox for interval length variable selection
        self.max_bxstr = tk.StringVar(value = 10)
        self.max_bx = ttk.Combobox(self.ws, textvariable=self.max_bxstr)
        #self.interval_bx.bind('<<ComboboxSelected>>', interval_chosen)
        self.max_bx['values'] = (10,50,100,1000)
        self.max_bx.config(width = 15)
        
        #if default data collection is selected, parameter selection is unnecessary
        if not enabled:
            self.max_bx.config(state=tk.DISABLED)
            
        #declare and initialize variables used to send data outside of the class
        self.end_dt = dt.datetime.now(dt.timezone.utc)
        self._datestr2 = ""
        self._interval_len = 0
        self._datestr = ""
        self._json_max = 0
        self._has_values = False
    def on_quit(self):
        print("Calendar was terminated by exiting the window")
        self.ws.destroy()
        
    def display_msg(self):
        #retrieve selected end date from calendar         
        self._datestr2 = self.end_cal.get_date()

        date2_r = self._datestr2.split("/")
        
        #adding the 2000 is bad practice, but this code will not exist in the year 3000 and twitter did not exist in 1999
        date2 = dt.date(int(date2_r[2])+2000, int(date2_r[0]), int(date2_r[1]))
        
        #retrieve timezone-aware time at exactly midnight UTC
        ti = dt.time(tzinfo = dt.timezone.utc)
        
        self.end_dt = dt.datetime.combine(date2,ti)
        
        #retrieve use selections from comboboxes
        self._interval_len = self.interval_bx.get()
        self._json_max = self.max_bx.get()

        
        t = f"Your query is for tweets created before {self._datestr2} with data collection intervals of length {self._interval_len} minutes."
        print(t)
 
        #check for illegal arguments 
        if float(self._interval_len) < 1:
            raise TwitterAPIArgumentError(f"Invalid time interval ({self._interval_len}) received")
        pluralizer = "s" if float(self._interval_len) > 1 else ""
        if float(self._json_max) < 1:
            raise TwitterAPIArgumentError(f"Invalid rate of tweets ({self._json_max}) per ({self._interval_len}) minute{pluralizer} requested. At least one tweet must be requested per time interval.")  

        self._has_values = True
        
        #once user selects search parameters, UI is no longer necesssary
        self.ws.quit()
    
    #getter methods for user input
    @property   
    def interval_len(self):
        return self._interval_len
    @property
    def datestr2(self):
        return self._datestr2
    @property
    def json_max(self):
        return self._json_max
    @property
    def has_values(self):
        return self._has_values
        
    def run_cal(self):
        self.ws.title("Set Date Range for Data Collection")
        self.ws.geometry("600x430")
        self.ws.config(bg="#cae7e8")
        self.ws.config(borderwidth=15)
        
        self.end_cal_label = tk.Label(
            self.ws, 
            text="Select End Date For Tweet Collection",
            font=("Times", 12),
            bg="#cae7e8"
            )
            
        self.end_cal_label.pack(side = tk.TOP, pady = 5)
        self.end_cal.pack(side = tk.TOP)

        actionBtn = tk.Button(
            self.ws,
            text="Run Query",
            padx=10,
            pady=10,
            command=self.display_msg
        )
        self.interval_bx.pack(side=tk.BOTTOM, expand=False, pady=2)
        self.msg.pack(side=tk.BOTTOM, pady=2)
        
        self.max_bx.pack(side=tk.BOTTOM, expand=False, pady=2)
        self.msg2.pack(side=tk.BOTTOM, pady=2)
        actionBtn.pack(side=tk.BOTTOM, pady=2)
        
        self.msg_display.pack(side=tk.BOTTOM, pady=3)
        
        self.ws.protocol('WM_DELETE_WINDOW', self.on_quit)
        self.ws.mainloop()



import requests
import json
import datetime as dt
import math
import pickle
from cleantext import clean
import re
import time
import twitsent.plot_sent as ps
import twitsent.parse_sentiment as pars
import twitsent.twitterquery as tq
import sys
import os
import twitsent.store_data as sd
import twitsent.dateselect as ds
from os import listdir
from os.path import isfile, join
import datetime as dt
# To set your environment variables in your (Linux) terminal run the following line:
# export 'BEARER_TOKEN'='<your_bearer_token>'

#hardcoded token for testing purposes
bearer_token = 'AAAAAAAAAAAAAAAAAAAAALpLJAEAAAAAwm%2B7SfQEhgbzuZ7af1nthlIt5Yw%3DfGN0INCr4TdAxUveVtV0KlPP5XwyEWK0IGzWAhnvCNT2B7DNFo'

search_url = "https://api.twitter.com/2/tweets/search/recent" #replace with "https://api.twitter.com/2/tweets/search/all" if you have academic access

class RateLimitError(Exception):
    def __init__(self, message):
        super().__init__(message)
        
class TwitterAPIArgumentError(Exception):
    def __init__(self, message):
        super().__init__(message)

class CalendarError(Exception):
    def __init__(self, message):
        super().__init__(message)
        
def bearer_oauth(r):
    """
    Method required by bearer token authentication.
    """
    r.headers["Authorization"] = f"Bearer {bearer_token}"
    r.headers["User-Agent"] = "v2RecentSearchPython"
    
    return r

def connect_to_endpoint(url, params):
    response = requests.get(url, auth=bearer_oauth, params=params)
    print(response.status_code)
    # if rate limit response code is received, wait 15 minutes until limit is reset
    if response.status_code == 429:
        print("Rate limit exceeded, waiting until request can be satisfied")
        time.sleep(900)
        print("Restarting query")
        response = requests.get(url, auth=bearer_oauth, params=params)
        print(response.status_code)
    if response.status_code == 429:
        raise RateLimitError("Rate limit was not reset after 15 minutes as expected, quitting")
    if response.status_code != 200:
        raise Exception(response.status_code, response.text)
    return response.json()

'''
INPUT: query_params- dictionary that serves as instructions for the twitter search api 
        json_max- max number of tweets to store as cleaned text per time interval
        totaltime- length of time in minutes between earliest and latest possible tweets in API requests
        interval_len- length of each distinct time interval for tweet retrieval in minutes
OUTPUT: list of lists that stores tweet texts for each interval of time 
'''
def create_timeseries(query_params, json_max, totaltime, interval_len, end_time_raw = dt.datetime.now(dt.timezone.utc)):
    
    #Twitter API request needs to be historical by at least 10 seconds
    offset_delta = dt.timedelta(seconds = 30)
    end_time_raw -= offset_delta   
    
    test_delta = dt.timedelta(minutes=1)
    
    json_count = 0 #stores number of tweets retrieved for each tine interval so far
    json_count_list = [] #stores final number of tweets collected per each time interval
    
    '''
    run test to see how many requests are likely needed to fulfill json_max requirement for a given historical period
    '''
    #calculate start and endpoints for one interval
    start_time_raw = end_time_raw - test_delta
    #convert to the timedate format that the twitter api needs
    start_time = start_time_raw.isoformat()
    end_time = end_time_raw.isoformat()
        
    #TODO retrieve twitter api search results for time interval
    query_params['start_time'] = start_time
    query_params['end_time'] = end_time
    
    #twitter search api v2 limits search results to 100 per request
    if json_max < 100:
        query_params['max_results'] = json_max
    else: 
        query_params['max_results'] = 100
        
    json_response = connect_to_endpoint(search_url, query_params) 
        
    '''
    count number of tweets found within the last minute 
    '''
    if 'data' in json_response:
        #extract tweet data fron json response line
        data = json_response["data"]
        #extract text from tweet list
        for tweet_inst in data:
            json_count += 1
                
        if 'next_token' in json_response:
            next_token = json_response["meta"]["next_token"]
        else:
            next_token = None
                
        while(next_token is not None):
            # construct a ruleset from all rules
            query_params['next_token'] = next_token
            query_params['max_results'] = 100
            json_response = connect_to_endpoint(search_url, query_params)
                
            json_count += 1
                
            if 'next_token' in json_response:
                next_token = json_response["meta"]["next_token"]
            else:
                next_token = None
    #no tweets were found in the last minute, assume that tweets about subject are rare
    else:
        json_count = 1
      
    '''
    retrieve actual data for historical period
    '''
    #how many intervals need to be queried
    interval_num = math.ceil(totaltime / interval_len)  
    if totaltime % interval_len != 0:
        print("Warning: One time interval is of unequal length to the others and will likely not have complete data. Please ensure that total time is divisible by interval length") 
        
    #estimate how many minutes interval_len needs to be to ensure adequate data is collected
    mins = json_max / json_count * 2
    if mins > interval_len:
        print("Warning: data collection will likely be incomplete due to short time intervals allotted for tweet collection or low tweet quantity")
        
    # number of queries within each interval likely needed to retrieve adequate data 
    requests = math.ceil(json_max / 100)
    
    #calculate how long of a time interval is allotted for each search request given the necessary number of requests per interval
    delta = dt.timedelta(minutes=interval_len)
    request_delta = delta/requests
    
    json_interval = [] #stores json tweet data for each time interval
    json_response_list = [] #stores all json interval data
    json_count = 0 #reset json_count variable after test
    end_copy = end_time_raw
    
    #for each interval
    for time_int in range(interval_num):
        #multiple requests are made per time interval due to twitter's limit (100) to the quantity of tweets retrieved per request
        for request in range(requests):
            #if the max number of tweets per interval has not yet been reached
            if json_count < json_max:
                #calculate start and endpoints for one request
                start_time_raw = end_time_raw - request_delta
                #convert to the timedate format that the twitter api needs
                start_time = start_time_raw.isoformat()
                end_time = end_time_raw.isoformat()
                
                query_params['start_time'] = start_time
                query_params['end_time'] = end_time
                
                #twitter search api v2 limits search results to 100 per request
                if json_max < 100:
                    query_params['max_results'] = json_max
                else: 
                    query_params['max_results'] = 100
                    
                json_response = connect_to_endpoint(search_url, query_params) 
                
                '''
                extract and clean useful data from tweet, then store it in a time-delimited array
                '''
                if 'data' in json_response:
                    #extract tweet data fron json response line
                    data = json_response["data"]
                    #extract text from tweet list
                    #twitter returns more than one tweet per request
                    for tweet_inst in data:
                        if json_count < json_max:
                            text = tweet_inst["text"]
                            #remove emojis and other symbols from tweet text
                            text = clean(text,
                                fix_unicode=True,               # fix various unicode errors
                                to_ascii=True,                  # transliterate to closest ASCII representation
                                lower=True,                     # lowercase text
                                no_line_breaks=True,           # fully strip line breaks as opposed to only normalizing them
                                no_urls=True,                  # replace all URLs with a special token
                                no_emails=True,                # replace all email addresses with a special token
                                no_phone_numbers=True,         # replace all phone numbers with a special token
                                no_numbers=True,               # replace all numbers with a special token
                                no_digits=True,                # replace all digits with a special token
                                no_currency_symbols=True,      # replace all currency symbols with a special token
                                no_punct=True,                 # remove punctuations
                                replace_with_url="",
                                replace_with_email="",
                                replace_with_phone_number="",
                                replace_with_number="",
                                replace_with_digit="",
                                replace_with_currency_symbol="",
                                no_emoji=True,
                                lang="en"                       # set to 'de' for German special handling
                                )
                            #remove retweet characters and end of line characters from tweet text
                            text = re.sub(r"\brt\b","",text)
                            #remove non-utf-8 characters from string
                            tweet = bytes(text, 'utf-8').decode('utf-8', 'ignore')
                            
                            tweet = str(tweet)
                            tweet = tweet.strip()
                               
                            #store data retrieved and paginate if necessary
                            json_interval.append(tweet)
                            json_count += 1
                else:
                    print("No matching tweets for time interval starting at " + start_time)
                if 'next_token' in json_response:
                    next_token = json_response["meta"]["next_token"]
                else:
                    next_token = None
            else:
                break
            #twitter requires you to interate through page requests if more tweets were found than fit in one response(up to a limit of 100 tweets total)
            while(next_token is not None):
                if json_count < json_max:
                    # construct a ruleset from all rules
                    query_params['next_token'] = next_token
                    #twitter search api v2 limits search results to 100 per request
                    if json_max < 100:
                        query_params['max_results'] = json_max
                    else: 
                        query_params['max_results'] = 100
                    json_response = connect_to_endpoint(search_url, query_params)
                    
                    '''
                    extract and clean useful data from tweet, then store it in a time-delimited array
                    '''
                    #extract tweet data fron json response line
                    data = json_response["data"] 
                    #extract text from tweet data
                    text = data["text"]
                    #remove emojis from tweet text
                    text = clean(text,
                        fix_unicode=True,               # fix various unicode errors
                        to_ascii=True,                  # transliterate to closest ASCII representation
                        lower=True,                     # lowercase text
                        no_line_breaks=True,           # fully strip line breaks as opposed to only normalizing them
                        no_urls=True,                  # replace all URLs with a special token
                        no_emails=True,                # replace all email addresses with a special token
                        no_phone_numbers=True,         # replace all phone numbers with a special token
                        no_numbers=True,               # replace all numbers with a special token
                        no_digits=True,                # replace all digits with a special token
                        no_currency_symbols=True,      # replace all currency symbols with a special token
                        no_punct=True,                 # remove punctuations
                        replace_with_url="",
                        replace_with_email="",
                        replace_with_phone_number="",
                        replace_with_number="",
                        replace_with_digit="",
                        replace_with_currency_symbol="",
                        no_emoji=True,
                        lang="en"                       # set to 'de' for German special handling
                        )
                    #remove retweet characters and end of line characters from tweet text
                    text = re.sub(r"\brt\b","",text)
                    #remove non-utf-8 characters from string
                    tweet = bytes(text, 'utf-8').decode('utf-8', 'ignore')
                    
                    tweet = str(tweet)
                    tweet = tweet.strip()
                    json_interval.append(tweet)
                    json_count += 1
                    
                    if 'next_token' in json_response:
                        next_token = json_response["meta"]["next_token"]
                    else:
                        next_token = None
                else:
                    next_token = None
        #update datetime endpoints with original undivided delta to ensure uniformity of each interval length is maintained(I am uncertain how the datetime python package rounds values when you perform operations on a timedelta object)
        end_time_raw = end_copy - delta
        end_copy  = end_time_raw

        json_response_list.append(json_interval.copy())
        json_count_list.append(json_count)
        json_count = 0
        json_interval.clear()
        
        
    return json_response_list
   
def list_extrema(senti_list, tweet_list):
    mini = 1.01
    maxi = -1.01
    mini_tweet = ""
    maxi_tweet = ""
    min_list = []
    max_list = []
    counter = 0
    for interval in senti_list:
        mini = 1.01
        maxi = -1.01
        mini_tweet = ""
        maxi_tweet = ""
        counter = 0
        for score in interval:
            if score > maxi:
                maxi = score
                maxi_tweet = tweet_list[counter]
            if score < mini:
                mini = score
                mini_tweet = tweet_list[counter]
        min_list.append(mini)
        max_list.append(maxi)
        counter += 1
            
'''
Retrieve tweets that contain certain keywords, parse sentiment scores from the text, then graph average sentiment over a specific timespan with certain intervals
'''    
def main():

    # Optional params: start_time,end_time,since_id,until_id,max_results,next_token,
    # expansions,tweet.fields,media.fields,poll.fields,place.fields,user.fields
    rule = None
    rule2 = [['the'],['i'],['to'],['a'],['and'],['is'],['in'],['it'],['you'],['of'],['for'],['on'],['my'],['that'],['at'],['with'],['me'],['do'],['have'],['just'],['this'],['be'],['so'],['are'],['not']] #list of 10 most common english words
   
    lang = ["en"]
    #construct the query for Twitter's search API v2
    #query_params = tq.make_query(rule, lang)
    query_params2 = tq.make_query(rule2, lang)
    
    #set default search and storage parameters
    start_dt = None
    end_dt = None
    datestr = ""
    datestr2 = ""
    newdatestr2 = ""
    academic_access = ""
    use_default = ""
    start_fresh = ""
    past_file = ""
    json_max = 10 #2500 tweets retrieved per hour maxes out 2M tweet per month limit
    interval_len = 240
    totaltime = 0
    exists_prevdata = True
    has_academic = False
    
    #get path to parent directory of this file
    rel_path = os.path.dirname(os.path.realpath(__file__))

    #construct the full path to store collected tweet data
    datadir = "storedqueries"
    mypath = os.path.join(rel_path, datadir)
          
    #make the data storage directory if it doesn't already exist
    os.makedirs(os.path.abspath(mypath), mode=0o666, exist_ok=True)
    
    onlyfiles = [f for f in listdir(mypath) if isfile(join(mypath, f))]

    date_c = []
    
    print("Do you have academic access to the Twitter search API? Y/N or Q to quit")
    while (academic_access.lower() != 'y' and academic_access.lower() != 'n' and academic_access.lower() != 'q'):
        academic_access = input()
    if academic_access.lower() == 'q':
        return 
        
    print("Use default data collection settings? Y/N or Q to quit")
    while (use_default.lower() != 'y' and use_default.lower() != 'n' and use_default.lower() != 'q'):
        use_default = input()
    if use_default.lower() == 'q':
        return
        

    print("Start new data collection? Y/N or Q to quit")
    while (start_fresh.lower() != 'y' and start_fresh.lower() != 'n' and start_fresh.lower() != 'q'):
        start_fresh = input()
    if start_fresh.lower() == 'q':
        return
    
    
    print("Enter a comma delimited list of search terms, D for default terms, or Q to quit")
    search_terms = input().lower()
    if search_terms == 'q':
        return
    if search_terms == 'd':
        rule = [["corona virus"],['coronavirus'],["corona", "-beer"], ["covid"], ['covid 19'], ['covid19'], ['covid-19'], ['sarscov2'], ['sars cov 2'], ['sars-cov-2'], ['#coronavirus'], ['#corona'], ['#covid'], ['#covid19'], ['#sarscov2']]
    else:
        #Create a rule array from user input
        terms = search_terms.split(",")# TODO watch out for malicious input here
        rule = [sub_term.split(" ") for sub_term in terms]
        
    query_params = tq.make_query(rule, lang)
    
    if(use_default.lower() == 'y'):
        if start_fresh.lower() == 'y':
            ci = ds.Cal_Impl(False)
            ci.run_cal()

            if( not ci.has_values): #check if parameters were chosen from the calendar, or if it was closed prematurely
                raise CalendarError("Calendar did not select values before quitting")
            #retrieve start and end dates from calendar
            datestr = ci.datestr
            datestr2 = ci.datestr2
            newdatestr2 = datestr2
            
            #construct datetime dates for query limits calculation
            date2_r = datestr2.split("/")  
            date2 = dt.date(int(date2_r[2])+2000, int(date2_r[0]), int(date2_r[1]))
                
            #retrieve timezone-aware time at exactly midnight
            ti = dt.time(tzinfo = dt.timezone.utc)
                
            end_dt = dt.datetime.combine(date2,ti)
              
            #retrieve query params from calendar 
            json_max = int(ci.json_max)
            interval_len = int(ci.interval_len)
            totaltime = ci.totaltime
            
            #calculate the time in minutes between the current time and the time of the first request
            delta_first_request = (dt.datetime.now(dt.timezone.utc)-end_dt)/dt.timedelta(minutes = 1)
            
            #ensure that users follow API request time limitations 
            if totaltime + delta_first_request >= 10080 and academic_access != 'y': #if user does not have academic access, they cannot request tweets more than a week in the past
                raise TwitterAPIArgumentError("Academic access to Twitter's API is required to search for tweets more than a week in the past")
            '''
            If starting fresh, delete past data
            '''
            #find past data storage files with matching search parameters
            file_matches = []
            for file in onlyfiles:
                file_data = file.split("_")
                file_prefix = file_data[0]
                if file_prefix == "":
                    continue
                file_json_max = int(file_data[3])
                file_interval_len = int(file_data[4])
                if ((file_prefix == 'tweet' or file_prefix == 'senti') and file_json_max == json_max and file_interval_len == interval_len):
                    file_matches.append(file)
                 
            #if previous data storage file exists, delete it
            for file in file_matches: 
                filepath = os.path.join(mypath,file)    
                if os.path.exists(filepath):
                    os.rename(filepath, os.path.join(mypath, "_archived_" + file))
                    #os.remove(filepath)
                else:
                    print(f"Cannot delete the file ({filepath}) because it does not exist")   
                    
        elif start_fresh.lower() == 'n':
            ci = ds.Cal_End(False)
            ci.run_cal()
            
            if( not ci.has_values): #check if parameters were chosen from the calendar, or if it was closed prematurely
                raise CalendarError("Calendar did not select values before quitting")
            #search in specified directory for file that matches default data storage file values (which are contained within the file name)
            for file in onlyfiles:
                file_data = file.split("_")
                file_prefix = file_data[0]
                if file_prefix == "":
                    continue
                file_startdate = file_data[1]
                datestr = file_startdate
                file_enddate = file_data[2]
                datestr2 = file_enddate
                date_c = file_enddate.split(".")
                file_json_max = int(file_data[3])
                file_interval_len = int(file_data[4])
                if ((file_prefix == 'tweet' or file_prefix == 'senti') and file_json_max == json_max and file_interval_len == interval_len):
                    past_file = file
                    break
            if past_file == "":
                print("No previous tweet data found")
                return
                
            newdatestr2 = ci.datestr2
            
            #construct datetime date for query end
            date2_r = newdatestr2.split("/")  
            end_date = dt.date(int(date2_r[2])+2000, int(date2_r[0]), int(date2_r[1]))
                
            #retrieve timezone-aware time at exactly midnight
            ti = dt.time(tzinfo = dt.timezone.utc)
              
            #date of last recorded entry in csv file
            prev_end_date = dt.date(int(date_c[2])+2000, int(date_c[0]), int(date_c[1])) #prev_end_date date of data collection will be end date of last data collection period
            
            #string representation of query end date for use in csv filenames
            newdatestr2 = str(end_date.month).lstrip("0") + "." + str(end_date.day).lstrip("0") + "." + str(end_date.year)[-2:] 
            
            #set query time constraints
            start_dt = dt.datetime.combine(prev_end_date,ti) #Note that start_dt does NOT correspond to the datetime when data collection first started, but the most recent data collection starting point!
            end_dt = dt.datetime.combine(end_date,ti)
            totaltime = int((end_dt - start_dt) / dt.timedelta(minutes = 1))
            
            #calculate the time in minutes between the current time and the time of the first request
            delta_first_request = (dt.datetime.now(dt.timezone.utc)-end_dt)/dt.timedelta(minutes = 1)
            
            #ensure that users follow API request time limitations 
            if totaltime + delta_first_request >= 10080 and academic_access != 'y': #if user does not have academic access, they cannot request tweets more than a week in the past
                raise TwitterAPIArgumentError("Academic access to Twitter's API is required to search for tweets more than a week in the past")

    elif(use_default.lower() == 'n'):      
        if start_fresh.lower() == 'y':
            ci = ds.Cal_Impl(True)
            ci.run_cal()
            
            if( not ci.has_values): #check if parameters were chosen from the calendar, or if it was closed prematurely
                raise CalendarError("Calendar did not select values before quitting")
            #retrieve start and end dates from calendar
            datestr = ci.datestr
            datestr2 = ci.datestr2
            newdatestr2 = datestr2
            
            #construct datetime dates for query limits calculation
            date2_r = datestr2.split("/")  
            date2 = dt.date(int(date2_r[2])+2000, int(date2_r[0]), int(date2_r[1]))
                
            #retrieve timezone-aware time at exactly midnight
            ti = dt.time(tzinfo = dt.timezone.utc)
                
            end_dt = dt.datetime.combine(date2,ti)
              
            #retrieve query params from calendar 
            json_max = int(ci.json_max)
            interval_len = int(ci.interval_len)
            totaltime = ci.totaltime
            
            #calculate the time in minutes between the current time and the time of the first request
            delta_first_request = (dt.datetime.now(dt.timezone.utc)-end_dt)/dt.timedelta(minutes = 1)
            
            #ensure that users follow API request time limitations 
            if totaltime + delta_first_request >= 10080 and academic_access != 'y': #if user does not have academic access, they cannot request tweets more than a week in the past
                raise TwitterAPIArgumentError("Academic access to Twitter's API is required to search for tweets more than a week in the past")
                
            '''
            If starting fresh, delete past data
            '''
            #find past data storage files with matching search parameters
            file_matches = []
            for file in onlyfiles:
                file_data = file.split("_")
                file_prefix = file_data[0]
                if file_prefix == "":
                    continue
                file_json_max = int(file_data[3])
                file_interval_len = int(file_data[4])
                if ((file_prefix == 'tweet' or file_prefix == 'senti') and file_json_max == json_max and file_interval_len == interval_len):
                    file_matches.append(file)
                 
            #if previous data storage file exists, delete it
            for file in file_matches: 
                filepath = os.path.join(mypath,file)    
                if os.path.exists(filepath):
                    os.rename(filepath, os.path.join(mypath, "_archived_" + file))
                    #os.remove(filepath)
                else:
                  print(f"Cannot delete the file ({filepath}) because it does not exist")   
        elif start_fresh.lower() == 'n':
            ci = ds.Cal_End(True)
            ci.run_cal()
            
            if( not ci.has_values): #check if parameters were chosen from the calendar, or if it was closed prematurely
                raise CalendarError("Calendar did not select values before quitting")
            #retrieve query params from calendar 
            json_max = int(ci.json_max)
            interval_len = int(ci.interval_len) 
                
            #search in specified directory for file that matches default data storage file values (which are contained within the file name)
            for file in onlyfiles:
                file_data = file.split("_")
                file_prefix = file_data[0]
                if file_prefix == "":
                    continue
                file_startdate = file_data[1]
                datestr = file_startdate
                file_enddate = file_data[2]
                datestr2 = file_enddate
                date_c = file_enddate.split(".")
                file_json_max = int(file_data[3])
                file_interval_len = int(file_data[4])
                if ((file_prefix == 'tweet' or file_prefix == 'senti') and file_json_max == json_max and file_interval_len == interval_len):
                    past_file = file
                    break
            if past_file == "":
                print("No previous tweet data found")
                return
            
            newdatestr2 = ci.datestr2
            
            #construct datetime dates for query limits calculation
            date2_r = newdatestr2.split("/")  
            end_date = dt.date(int(date2_r[2])+2000, int(date2_r[0]), int(date2_r[1]))
                
            #retrieve timezone-aware time at exactly midnight
            ti = dt.time(tzinfo = dt.timezone.utc)      
            
            prev_end_date = dt.date(int(date_c[2])+2000, int(date_c[0]), int(date_c[1])) #prev_end_date date of data collection will be end date of last data collection period
            
            #string representation of query end date for use in csv filenames
            newdatestr2 = str(end_date.month).lstrip("0") + "." + str(end_date.day).lstrip("0") + "." + str(end_date.year)[-2:] 
            
            #set query time constraints
            start_dt = dt.datetime.combine(prev_end_date,ti) #Note that start_dt does NOT correspond to the datetime when data collection first started, but the most recent data collection starting point!
            end_dt = dt.datetime.combine(end_date,ti)
            totaltime = int((end_dt - start_dt) / dt.timedelta(minutes = 1))
            
            #calculate the time in minutes between the current time and the time of the first request
            delta_first_request = (dt.datetime.now(dt.timezone.utc)-end_dt)/dt.timedelta(minutes = 1)
            
            #ensure that users follow API request time limitations 
            if totaltime + delta_first_request >= 10080 and academic_access != 'y': #if user does not have academic access, they cannot request tweets more than a week in the past
                raise TwitterAPIArgumentError("Academic access to Twitter's API is required to search for tweets more than a week in the past")
        
    #check for illegal arguments 
    if interval_len < 1:
        raise TwitterAPIArgumentError(f"Invalid time interval ({interval_len}) received")
    if totaltime < 1:
        raise TwitterAPIArgumentError(f"Invalid duration of search ({totaltime}) received. Ensure that start date and end date arguments of data collection are sequential.")
    pluralizer = "s" if interval_len > 1 else ""
    if json_max < 1:
        raise TwitterAPIArgumentError(f"Invalid rate of tweets ({json_max}) per ({interval_len}) minute{pluralizer} requested. At least one tweet must be requested per time interval.")
    
    #retrieve tweet data for each time interval within the total time queried
    json_response_list = create_timeseries(query_params, json_max, totaltime, interval_len, end_dt)  
    json_response_list2 = create_timeseries(query_params2, json_max, totaltime, interval_len, end_dt)
    
    #convert tweet text list into sentiment score list
    sentiment_list = pars.parse(json_response_list)
    sentiment_list2 = pars.parse(json_response_list2)
    
    #save tweet data collected for later use
    sd.save_lists(json_response_list, json_response_list2, sentiment_list, sentiment_list2, datestr, datestr2, newdatestr2, json_max, interval_len)
    
    #load all historical data for graphing if the user desires
    sentiment_list, _, sentiment_list2, _, totaltime = sd.load_lists(json_max, interval_len)

    #take mean of sentiment scores for each interval, using zero for the mean of any intervals that have no scores due to lack of data
    avg_sent = [sum(interval)/len(interval) if len(interval) != 0 else 0 for interval in sentiment_list]
    comp_sent = [sum(interval)/len(interval) if len(interval) != 0 else 0 for interval in sentiment_list2]
    
    #graph sentiment data 
    ps.sent_line(avg_sent, comp_sent, totaltime, interval_len)

    
if __name__ == "__main__":
    main()



import matplotlib.pyplot as plt
import math
import pymannkendall as mk
import pickle

'''
Given a list containing sentiment scores collected at specific intervals,
this method draws a line-graph and tests for trends in the data
INPUT:  sent_array- list of sentiment scores, in float format
        totaltime- time in minutes that data is collected over(int)
        interval_len- length of each interval that data is collected from
OUTPUT: None
RESULTS: Draws a line graph and displays significance score of sent_array timeseries data
'''
class TwitterAPIArgumentError(Exception):
    def __init__(self, message):
        super().__init__(message)
        
def sent_line(sent_array, sent_array_neu, totaltime, interval_len):
    
    if interval_len < 1:
        raise TwitterAPIArgumentError(f"Invalid time interval ({interval_len}) received")
    if totaltime < 1:
        raise TwitterAPIArgumentError(f"Invalid duration of search ({totaltime}) received. Ensure that start date and end date arguments of data collection are sequential.")
    time_list = [] #list of cumulative times elapsed since start time for each interval, in minutes 
    is_sig = ""
    num_intervals = math.ceil(totaltime/interval_len)
    time_elapsed = 0
    try:
        #calculate statistical significance of timeseries data without the assumption of gaussian distribution
        stat_sig = mk.original_test(sent_array, alpha = .05)
    except ZeroDivisionError:
        print("Time intervals of data collection exceed duration of data collection")

    #create x axis timeseries data for graphing, each datapoint spaced interval_len apart
    for i in range(num_intervals):
        #make sure that a smaller interval at the end(assuming total time isn't divided evenly by interval_len) isn't allotted the max interval_len
        time_list.append(min(time_elapsed,totaltime))
        time_elapsed += interval_len
    
    if stat_sig[1] is True:
        is_sig = 'statistically signicant'
    else:
        is_sig = 'statistically insignicant'
        
    #create a string that informs users about the statistical significance of any trend observed in the data
    stat_string = 'The trendline y = ' + str(round(stat_sig[7], 2)) + 'x' + ' + ' + str(round(stat_sig[8], 2)) + ' is ' + is_sig + '. The p-value of this particular \ntimeseries using the Mann-Kendall test is ' + str(round(stat_sig[2], 2)) + '.'

    #plt.figure(figsize=(8,6))
    fig, ax = plt.subplots(figsize=(8, 6))
    #entries in the sent_array list are in reverse chronological order
    ax.invert_xaxis()
    plt.gcf().subplots_adjust(bottom=0.2)
    plt.plot(time_list, sent_array, color = 'blue', marker = 'o', label="Search Term Sentiment")
    plt.plot(time_list, sent_array_neu, color = 'red', marker = 'o', label="Baseline Tweet Sentiment")
    plt.legend()
    plt.title('Sentiment Over Time', pad = 20, fontsize=24)
    plt.xlabel('Minutes Ago', fontsize=14)
    plt.ylabel('Average Sentiment', fontsize=14)
    plt.grid(True)
    plt.ylim((-1,1))
    plt.annotate(stat_string, xy = (0,-.2), fontsize=10,xycoords='axes fraction',ha='left') 
    plt.savefig('sentiment_comparisongraph.png', bbox_inches='tight')
    plt.show()
        
'''
Hussain et al., (2019). pyMannKendall: a python package for non parametric Mann Kendall family of trend tests.. Journal of Open Source Software, 4(39), 1556, https://doi.org/10.21105/joss.01556
'''


# import module
import codecs
import webbrowser
import os

# to open/create a new html file in the write mode
f = open('Results-Page-TweetSA.html', 'w')

#directory of python script as variable      
#get path to parent directory of this file
rel_path = os.path.dirname(os.path.realpath(__file__))
#construct the full path to store collected tweet data
filename = "sentiment_comparisongraph.png"
mypath = os.path.join(rel_path, filename)

# the html code which will go in the file Results-Page-TweetSA.html
html_template = f"""

<html lang="en-US:>

<head>
<title>Tweet Sentiment Analysis Student Research</title>
<basefont size=4>
</head>

<body bgcolor=FFFFFF>

<h1 align=center>Twitter Sentiment Analysis With NLTK's Vader</h1>
<h2 align=center>(ACO 499) 2022 Summer Session B</h6>

<h2 align=center>Mitchell Hoikka</h6>

<p align=center>Display and analyse twitter sentiment data for given keywords</p>

<p align=center>Execute Python code and prompt user input</p>



<link rel="stylesheet" href="https://pyscript.net/alpha/pyscript.css" />
<script defer src="https://pyscript.net/alpha/pyscript.js"></script>

<img src={mypath} alt="Graph comparing baseline sentiment to keyword sentiment scores">
<hr>

<h2>How about a nice ordered list!</h2>
<ol>
  <li>This little piggy went to market
  <li>This little piggy went to SB228 class
  <li>This little piggy went to an expensive restaurant in Downtown Palo Alto
  <li>This little piggy ate too much at Indian Buffet.
  <li>This little piggy got lost
</ol>

<h2>Unordered list</h2>
<ul>
  <li>First element
  <li>Second element
  <li>Third element
</ul>

<hr>

<h2>Nested Lists!</h2>
<ul>
  <li>Things to to today:
    <ol>
      <li>Walk the dog
      <li>Feed the cat
      <li>Mow the lawn
    </ol>
  <li>Things to do tomorrow:
    <ol>
      <li>Lunch with mom
      <li>Feed the hamster
      <li>Clean kitchen
    </ol>
</ul>

<p>And finally, how about some <a href=http://www.yahoo.com/>Links?</a></p>

<p>Or let's just link to <a href=../../index.html>another page on this server</a></p>

<p>Remember, you can view the HTMl code from this or any other page by using the "View Page Source" command of your browser.</p>

</body>

</html>

"""

# writing the code into the file
f.write(html_template)

# close the file
f.close()

# viewing html files
# below code creates a
# codecs.StreamReaderWriter object
file = codecs.open("Results-Page-TweetSA.html", 'r', "utf-8")

# using .read method to view the html
# code from our object
print(file.read())

# open html file
webbrowser.open('Results-Page-TweetSA.html')



'''
This method constructs a query_param string for twitter api
INPUT: keywords_ops- list of lists. Each element in the outer list represent a condition separated by the 'OR' operator from the other elements. Each element in the 
        inner lists represents a condition separated by the 'AND' operator from the other elements in the same list.
        lang- list that stores language abbreviations that represent which languages tweets may be, with an empty list representing any language
OUTPUT: query_params- dictionary recognmized as a valid query format by the Twitter web API v2
RESULT: Twitter API v2 can be accessed in order to retrieve tweet data on keywords over a given interval
'''
def make_query(keywords_ops, lang):
    
    #remove list entries that contain no meaningful characters or contain forbidden characters
    keywords_ops = [[i for i in sublist if i.strip() != '' and "\\" not in  i and ":" not in i] for sublist in keywords_ops]
    #remove empty sublists
    keywords_ops = [sublist for sublist in keywords_ops if sublist]
    rule_list = []
    
    #list of legal twitter language abbreviations
    lang_list = ['en','ar','bn','cs','da','de','el','es','fa','fi','fil','fr','he','hi','hu','id','it','ja','ko','msa','nl','no','pl','pt','ro','ru','sv','th','tr','uk','ur','vi','zh-c','zh-tw']  
    temp_string = ""
    first = True #boolean that represents whether current sublist is the first in the list
    first_lang = True #boolean that represents whether the current language abbr. is the first to be added to the temp_string
    
    #construct twitter query string in correct syntax
    for possible_match in keywords_ops:
        if not first:
             temp_string += "OR "
        temp_string += "("
        for partial_match in possible_match:
            if temp_string != "(" and temp_string != "OR (":
                temp_string += " "
            if not "-" in partial_match:
                temp_string += "\"" + partial_match + "\""
            else:
                temp_string += partial_match 
        for abbr in lang:
            if abbr in lang_list:
                if first_lang:
                    temp_string += " ("
                if not first_lang:
                    temp_string += " OR "
                temp_string += "lang:" + str(abbr) 
                first_lang = False
        #checks if parenthesis for language filter need to be closed
        if not first_lang:
            temp_string += ")"
        temp_string += ")"
        rule_list.append(temp_string)
        temp_string = ""
        first = False
        first_lang = True
        
    # construct a ruleset from all rules
    rules = " ".join(rule_list)
    query_params = {'query': rules}
    
    return query_params 